@@ -5,6 +5,12 @@ for _, arg in ipairs({...}) do
     end
 end
 
+-- Sanity check
+
+if (not WANT_STRIPPED_BINARIES) or (WANT_STRIPPED_BINARIES == "") then
+    WANT_STRIPPED_BINARIES = "yes"
+end
+
 local allbinaries = {}
 local outfp = io.open(BUILDFILE, "w")
 local function emit(...)
@@ -24,7 +30,8 @@ local function has_package(package)
     if package == "builtin" then
         return true
     end
-    return os.execute("pkg-config "..package) == 0
+    local r = os.execute("pkg-config "..package)
+    return (r == 0) or (r == true)
 end
 
 local function detect_package(name, package)
@@ -39,8 +46,8 @@ end
 local function detect_mandatory_package(name, package)
     if not detect_package(name, package) then
         print()
-        print("Mandatory package is missing --- cannot build. (There's probably a built-")
-        print("in one; try 'builtin'.")
+        print("Mandatory package '"..name.."' is missing --- cannot build.")
+        print("(There's probably a built-in one; try 'builtin'.")
         print()
         os.exit(1)
     end
@@ -69,7 +76,7 @@ local function package_flags(package, kind)
 
     local filename = os.tmpname()
     local e = os.execute("pkg-config "..kind.." "..package.." > "..filename)
-    if e ~= 0 then
+    if (e ~= 0) and (e ~= true) then
         error("required package "..package.." is not available")
     end
     local s = io.open(filename):read("*a")
@@ -122,7 +129,7 @@ function build_wordgrinder_binary(exe, luapackage, frontend, buildstyle)
         ldflags[#ldflags+1] = "-lcomdlg32"
     end
 
-    if (buildstyle == "static") or (frontend == "windows") then
+    if (buildstyle == "static") or (frontend == "windows") or (frontend == "cwindows") then
         cflags[#cflags+1] = "-DEMULATED_WCWIDTH"
     end
 
@@ -130,23 +137,18 @@ function build_wordgrinder_binary(exe, luapackage, frontend, buildstyle)
         cflags[#cflags+1] = "-O0"
     else
         cflags[#cflags+1] = "-Os"
+        cflags[#cflags+1] = "-DNDEBUG"
     end
 
     if luapackage == "builtin" then
         cflags[#cflags+1] = "-Isrc/c/emu/lua-5.1.5"
-        cflags[#cflags+1] = "-DLUA_USE_MKSTEMP"
+        cflags[#cflags+1] = "-DWINSHIM"
+        cflags[#cflags+1] = "-DLUA_USE_EMU_TMPNAM"
     else
         cflags[#cflags+1] = package_flags(luapackage, "--cflags")
         ldflags[#ldflags+1] = package_flags(luapackage, "--libs")
     end
 
-    if LUAFILESYSTEM_PACKAGE == "builtin" then
-        cflags[#cflags+1] = "-Isrc/c/emu/lfs"
-    else
-        cflags[#cflags+1] = package_flags(LUAFILESYSTEM_PACKAGE, "--cflags")
-        ldflags[#ldflags+1] = package_flags(LUAFILESYSTEM_PACKAGE, "--libs")
-    end
-
     if LUABITOP_PACKAGE == "builtin" then
         cflags[#cflags+1] = "-Isrc/c/emu/luabitop"
     else
@@ -154,9 +156,9 @@ function build_wordgrinder_binary(exe, luapackage, frontend, buildstyle)
         ldflags[#ldflags+1] = package_flags(LUABITOP_PACKAGE, "--libs")
     end
 
-	if UTHASH_PACKAGE == "builtin" then
-		cflags[#cflags+1] = "-Isrc/c/emu/uthash"
-	else
+    if UTHASH_PACKAGE == "builtin" then
+        cflags[#cflags+1] = "-Isrc/c/emu/uthash"
+    else
         cflags[#cflags+1] = package_flags(UTHASH_PACKAGE, "--cflags")
         ldflags[#ldflags+1] = package_flags(UTHASH_PACKAGE, "--libs")
     end
@@ -171,13 +173,25 @@ function build_wordgrinder_binary(exe, luapackage, frontend, buildstyle)
         cflags[#cflags+1] = "-mwindows"
         ldflags[#ldflags+1] = "-static"
         ldflags[#ldflags+1] = "-lcomctl32"
-	ldflags[#ldflags+1] = "-mwindows"
+        ldflags[#ldflags+1] = "-mwindows"
+        if (buildstyle == "release") and (WANT_STRIPPED_BINARIES == "yes") then
+            ldflags[#ldflags+1] = "-s"
+        end
+    elseif frontend == "cwindows" then
+        cc = WINCC
+        cflags[#cflags+1] = "-DARCH='\"windows\"'"
+        cflags[#cflags+1] = "-DWIN32"
+        cflags[#cflags+1] = "-DWINVER=0x0501"
+        cflags[#cflags+1] = "-Dmain=appMain"
+        cflags[#cflags+1] = "-mconsole"
+        ldflags[#ldflags+1] = "-static"
+        ldflags[#ldflags+1] = "-mconsole"
+        if (buildstyle == "release") and (WANT_STRIPPED_BINARIES == "yes") then
+            ldflags[#ldflags+1] = "-s"
+        end
     else
         cc = CC
         cflags[#cflags+1] = "-DARCH='\"unix\"'"
-        cflags[#cflags+1] = "-D_XOPEN_SOURCE_EXTENDED"
-        cflags[#cflags+1] = "-D_XOPEN_SOURCE"
-        cflags[#cflags+1] = "-D_GNU_SOURCE"
     end
 
     if MINIZIP_PACKAGE == "builtin" then
@@ -187,6 +201,13 @@ function build_wordgrinder_binary(exe, luapackage, frontend, buildstyle)
         ldflags[#ldflags+1] = package_flags(MINIZIP_PACKAGE, "--libs")
     end
 
+    if LPEG_PACKAGE == "builtin" then
+        cflags[#cflags+1] = "-Isrc/c/emu/lpeg"
+    else
+        cflags[#cflags+1] = package_flags(LPEG_PACKAGE, "--cflags")
+        ldflags[#ldflags+1] = package_flags(LPEG_PACKAGE, "--libs")
+    end
+
     local function srcfile(cfile)
         ofile = cfile:gsub("^(.*)%.c$", OBJDIR.."/"..name.."/%1.o")
         objs[#objs+1] = ofile
@@ -198,6 +219,7 @@ function build_wordgrinder_binary(exe, luapackage, frontend, buildstyle)
     -- Main core
 
     srcfile("src/c/utils.c")
+    srcfile("src/c/filesystem.c")
     srcfile("src/c/zip.c")
     srcfile("src/c/main.c")
     srcfile("src/c/lua.c")
@@ -207,11 +229,7 @@ function build_wordgrinder_binary(exe, luapackage, frontend, buildstyle)
 
     -- Additional optional libraries
 
-    if LUAFILESYSTEM_PACKAGE == "builtin" then
-        srcfile("src/c/emu/lfs/lfs.c")
-    end
-
-    if (buildstyle == "static") or (frontend == "windows") then
+    if (buildstyle == "static") or (frontend == "windows") or (frontend == "cwindows") then
         srcfile("src/c/emu/wcwidth.c")
     end
 
@@ -251,6 +269,8 @@ function build_wordgrinder_binary(exe, luapackage, frontend, buildstyle)
         srcfile("src/c/emu/lua-5.1.5/lundump.c")
         srcfile("src/c/emu/lua-5.1.5/lvm.c")
         srcfile("src/c/emu/lua-5.1.5/lzio.c")
+        srcfile("src/c/emu/lua-5.1.5/winshim.c")
+        srcfile("src/c/emu/tmpnam.c")
     end
 
     -- Frontends
@@ -265,6 +285,9 @@ function build_wordgrinder_binary(exe, luapackage, frontend, buildstyle)
         srcfile("src/c/arch/win32/gdi/glyphcache.c")
         srcfile("src/c/arch/win32/gdi/realmain.c")
         objs[#objs+1] = OBJDIR.."/wordgrinder.rc.o"
+    elseif frontend == "cwindows" then
+        srcfile("src/c/arch/win32/console/dpy.c")
+        srcfile("src/c/arch/win32/console/realmain.c")
     end
 
     -- Minizip
@@ -275,27 +298,48 @@ function build_wordgrinder_binary(exe, luapackage, frontend, buildstyle)
         srcfile("src/c/emu/minizip/unzip.c")
     end
 
+    if LPEG_PACKAGE == "builtin" then
+        srcfile("src/c/emu/lpeg/lpvm.c")
+        srcfile("src/c/emu/lpeg/lpcap.c")
+        srcfile("src/c/emu/lpeg/lptree.c")
+        srcfile("src/c/emu/lpeg/lpcode.c")
+        srcfile("src/c/emu/lpeg/lpprint.c")
+    end
+
     emit("build ", exe, ": ld ", table.concat(objs, " "))
     emit("  ldflags = ", table.concat(ldflags, " "))
     emit("  cc = ", cc)
 end
 
-function run_wordgrinder_tests(exe, luapackage, frontend, buildstyle)
+function run_wordgrinder_tests(exe, luapackage, frontend, buildstyle, noauto)
     name = package_name(luapackage).."-"..frontend.."-"..buildstyle
     exe = addname(exe, name)
-    allbinaries[#allbinaries+1] = "test-"..name
+    if not noauto then
+        allbinaries[#allbinaries+1] = "test-"..name
+    end
 
     local alltests = {}
     for _, test in ipairs({
         "tests/apply-markup.lua",
+        "tests/argument-parser.lua",
         "tests/change-paragraph-style.lua",
         "tests/clipboard.lua",
         "tests/delete-selection.lua",
         "tests/escape-strings.lua",
+        "tests/export-to-html.lua",
+        "tests/export-to-latex.lua",
+        "tests/export-to-markdown.lua",
+        "tests/export-to-opendocument.lua",
         "tests/export-to-text.lua",
+        "tests/export-to-troff.lua",
+        "tests/filesystem.lua",
         "tests/find-and-replace.lua",
         "tests/get-style-from-word.lua",
         "tests/immutable-paragraphs.lua",
+        "tests/import-from-html.lua",
+        "tests/import-from-opendocument.lua",
+        "tests/import-from-text.lua",
+        "tests/import-from-markdown.lua",
         "tests/insert-space-with-style-hint.lua",
         "tests/io-open-enoent.lua",
         "tests/line-down-into-style.lua",
@@ -306,21 +350,26 @@ function run_wordgrinder_tests(exe, luapackage, frontend, buildstyle)
         "tests/load-0.3.3.lua",
         "tests/load-0.4.1.lua",
         "tests/load-0.5.3.lua",
-        "tests/load-0.6.lua",
         "tests/load-0.6-v6.lua",
+        "tests/load-0.6.lua",
+        "tests/load-0.7.2.lua",
         "tests/load-failed.lua",
         "tests/move-while-selected.lua",
+        "tests/numbered-lists.lua",
         "tests/parse-string-into-words.lua",
         "tests/save-format-escaped-strings.lua",
         "tests/simple-editing.lua",
         "tests/smartquotes-selection.lua",
         "tests/smartquotes-typing.lua",
         "tests/spellchecker.lua",
+        "tests/tableio.lua",
         "tests/type-while-selected.lua",
         "tests/undo.lua",
+        "tests/utf8.lua",
         "tests/utils.lua",
         "tests/weirdness-cannot-save-settings.lua",
         "tests/weirdness-combining-words.lua",
+        "tests/weirdness-delete-word.lua",
         "tests/weirdness-deletion-with-multiple-spaces.lua",
         "tests/weirdness-end-of-lines.lua",
         "tests/weirdness-globals-applied-on-startup.lua",
@@ -354,12 +403,6 @@ function install_file(mode, src, dest)
     installables[#installables+1] = dest
 end
 
--- Sanity check
-
-if (not WANT_STRIPPED_BINARIES) or (WANT_STRIPPED_BINARIES == "") then
-    WANT_STRIPPED_BINARIES = "yes"
-end
-
 -- Detect what tools we have available.
 
 io.write("Windows toolchain: ")
@@ -367,14 +410,17 @@ if has_binary(WINCC) and has_binary(WINDRES) and has_binary(MAKENSIS) then
     print("found")
     FRONTENDS["windows"] = true
 else
-    print("not found")
+    print(string.format("not found (WINCC=%s, WINDRES=%s, MAKENSIS=%s)",
+        tostring(has_binary(WINCC)),
+        tostring(has_binary(WINDRES)),
+        tostring(has_binary(MAKENSIS))))
 end
 
 FRONTENDS["curses"] = detect_package("Curses", CURSES_PACKAGE)
 FRONTENDS["x11"] = detect_package("FreeType2", "freetype2") and detect_package("Xft", XFT_PACKAGE)
 
 detect_mandatory_package("Minizip", MINIZIP_PACKAGE)
-detect_mandatory_package("LuaFileSystem", LUAFILESYSTEM_PACKAGE)
+detect_mandatory_package("lpeg", LPEG_PACKAGE)
 detect_mandatory_package("uthash", UTHASH_PACKAGE)
 detect_mandatory_package("LuaBitOp", LUABITOP_PACKAGE)
 
@@ -405,9 +451,9 @@ if want_frontend("curses") then
 end
 if want_frontend("x11") then
     emit("X11_CFLAGS = ", package_flags("freetype2", "--cflags"),
-	" ", package_flags(XFT_PACKAGE, "--cflags"))
+    " ", package_flags(XFT_PACKAGE, "--cflags"))
     emit("X11_LDFLAGS = ", package_flags("freetype2", "--libs"),
-	" ",  package_flags(XFT_PACKAGE, "--libs"))
+    " ",  package_flags(XFT_PACKAGE, "--libs"))
 end
 emit("LUA_INTERPRETER = ", LUA_INTERPRETER)
 emit("WINDRES = ", WINDRES)
@@ -471,6 +517,7 @@ emit("build ", OBJDIR.."/luascripts.c: luascripts ", table.concat({
     "src/lua/import/html.lua",
     "src/lua/import/text.lua",
     "src/lua/import/opendocument.lua",
+    "src/lua/import/markdown.lua",
     "src/lua/navigate.lua",
     "src/lua/addons/goto.lua",
     "src/lua/addons/autosave.lua",
@@ -486,8 +533,14 @@ emit("build ", OBJDIR.."/luascripts.c: luascripts ", table.concat({
     "src/lua/addons/smartquotes.lua",
     "src/lua/addons/undo.lua",
     "src/lua/addons/spillchocker.lua",
+    "src/lua/addons/templates.lua",
+    "src/lua/addons/directories.lua",
+    "src/lua/addons/recents.lua",
     "src/lua/menu.lua",
     "src/lua/cli.lua",
+    "src/lua/lunamark/util.lua",
+    "src/lua/lunamark/entities.lua",
+    "src/lua/lunamark/markdown.lua",
 }, " "))
 
 if want_frontend("x11") or want_frontend("curses") then
@@ -546,14 +599,23 @@ if want_frontend("x11") or want_frontend("curses") then
         install_file("755", preferred_x11, DESTDIR..BINDIR.."/xwordgrinder")
     end
     install_file("644", "bin/wordgrinder.1", DESTDIR..MANDIR.."/man1/wordgrinder.1")
+    install_file("644", "bin/xwordgrinder.1", DESTDIR..MANDIR.."/man1/xwordgrinder.1")
     install_file("644", "README.wg", DESTDIR..DOCDIR.."/wordgrinder/README.wg")
+    install_file("644", "extras/wordgrinder.desktop", DESTDIR..SHAREDIR.."/applications/wordgrinder.desktop")
+    install_file("644", "extras/wordgrinder.mime", DESTDIR..SHAREDIR.."/mime-info/wordgrinder.mime")
+    install_file("644", "extras/icon.png", DESTDIR..SHAREDIR.."/pixmaps/wordgrinder.png")
 
     emit("build bin/wordgrinder.1: manpage wordgrinder.man")
     emit("  date = ", DATE)
     emit("  version = ", VERSION)
 
-    emit("build binaries: phony bin/wordgrinder.1 ", preferred_curses or "", " ", preferred_x11 or "", " ")
-	emit("build tests: phony ", preferred_test)
+    emit("build bin/xwordgrinder.1: manpage xwordgrinder.man")
+    emit("  date = ", DATE)
+    emit("  version = ", VERSION)
+
+    emit("build binaries: phony bin/wordgrinder.1 bin/xwordgrinder.1 ",
+        preferred_curses or "", " ", preferred_x11 or "", " ")
+    emit("build tests: phony ", preferred_test)
     emit("build all: phony binaries tests")
     emit("build install: phony all ", table.concat(installables, " "))
     emit("build install-notests: phony binaries ", table.concat(installables, " "))
@@ -562,14 +624,21 @@ end
 if want_frontend("windows") then
     for _, buildstyle in ipairs({"release", "debug"}) do
         build_wordgrinder_binary("bin/wordgrinder.exe", "builtin", "windows", buildstyle)
+        build_wordgrinder_binary("bin/wordgrinder.exe", "builtin", "cwindows", buildstyle)
     end
 
     emit("build ", OBJDIR, "/wordgrinder.rc.o: rcfile src/c/arch/win32/wordgrinder.rc | src/c/arch/win32/manifest.xml")
 
     local installer = "bin/WordGrinder-"..VERSION.."-setup.exe"
     allbinaries[#allbinaries+1] = installer
-    emit("build ", installer, ": makensis extras/windows-installer.nsi | bin/wordgrinder-builtin-windows-release.exe")
+    emit("build ", installer, ": makensis extras/windows-installer.nsi | bin/wordgrinder-builtin-windows-release.exe bin/wordgrinder-builtin-cwindows-release.exe")
+
+    emit("build wintests: phony test-builtin-cwindows-debug")
+    run_wordgrinder_tests("bin/wordgrinder.exe", "builtin", "cwindows", "debug", true)
 end
 
 emit("build clean: phony")
 emit("build dev: phony ", table.concat(allbinaries, " "))
+
+-- vim: sw=4 ts=4 et
+
