@@ -37,16 +37,24 @@ static int defaultattr = 0;
 
 static int cursorx = 0;
 static int cursory = 0;
+static bool cursorshown = true;
 
 static bool isfullscreen = false;
 static bool window_geometry_valid = false;
 static RECT window_geometry;
 static bool window_created = false;
 
+static int shadowwidth;
+static int shadowheight;
+static HBITMAP shadow = NULL;
+static WINDOWPLACEMENT nonFullScreenSize = { sizeof(WINDOWPLACEMENT) };
+
+static void resize_shadow(int width, int height);
 static void resize_buffer(bool force);
 static void fullscreen_cb(void);
 static void switch_to_full_screen(void);
 static void switch_to_windowed(void);
+static void create_window(void);
 
 static COLORREF custom_colours[16];
 
@@ -153,6 +161,7 @@ static void unicode_key(uni_t key, unsigned flags)
 		dpy_queuekey(-(VKM_CTRLASCII | key));
 		return;
 	}
+
 	if ((key == ' ') && (GetKeyState(VK_CONTROL) & 0x8000))
 	{
 		dpy_queuekey(-(VKM_CTRLASCII | 0));
@@ -164,61 +173,72 @@ static void unicode_key(uni_t key, unsigned flags)
 
 static bool special_key(int vk, unsigned flags)
 {
-	switch (vk)
+	if (GetKeyState(VK_LMENU) & 0x8000)
 	{
-		case VK_RETURN:
-			if (flags & (1<<29))
-			{
-				fullscreen_cb();
-				return true;
-			}
-			break;
-
-		case VK_SHIFT:
-		case VK_CONTROL:
-		case VK_CAPITAL:
-		case VK_MENU:
-		case VK_LWIN:
-		case VK_RWIN:
-		case VK_SNAPSHOT:
-		case VK_PAUSE:
-			return false;
-	}
-
-	if (vk > 0x90)
-		return false;
-
-	/* Numeric keypad. */
-	if ((vk >= 0x60) && (vk <= 0x6f))
-		return false;
-
-	if ((vk == ' ') || isdigit(vk) || isupper(vk))
-	{
-		if (flags & (1<<29))
+		if (vk == VK_RETURN)
+		{
+			fullscreen_cb();
+			return true;
+		}
+		else if ((vk == ' ') || isdigit(vk) || isupper(vk))
 		{
-			/* ALT pressed */
 			dpy_queuekey(-27);
 			dpy_queuekey(vk);
 			return true;
 		}
-
-		return false;
 	}
 
-	if (GetKeyState(VK_CONTROL) & 0x8000)
-		vk |= VKM_CTRL;
-	if (GetKeyState(VK_SHIFT) & 0x8000)
-		vk |= VKM_SHIFT;
+	switch (vk)
+	{
+		case VK_DOWN:
+		case VK_UP:
+		case VK_LEFT:
+		case VK_RIGHT:
+		case VK_HOME:
+		case VK_END:
+		case VK_BACK:
+		case VK_DELETE:
+		case VK_INSERT:
+		case VK_NEXT:
+		case VK_PRIOR:
+		case VK_TAB:
+		case VK_RETURN:
+		case VK_ESCAPE:
+		case VK_F1:
+		case VK_F2:
+		case VK_F3:
+		case VK_F4:
+		case VK_F5:
+		case VK_F6:
+		case VK_F7:
+		case VK_F8:
+		case VK_F9:
+		case VK_F10:
+		case VK_F11:
+		case VK_F12:
+			if (GetKeyState(VK_CONTROL) & 0x8000)
+				vk |= VKM_CTRL;
+			if (GetKeyState(VK_SHIFT) & 0x8000)
+				vk |= VKM_SHIFT;
 
-	dpy_queuekey(-vk);
-	return true;
+			dpy_queuekey(-vk);
+			return true;
+	}
+
+	return false;
 }
 
-static void paint_cb(HWND window, PAINTSTRUCT* ps, HDC dc)
+static void paint_cb(HWND window, PAINTSTRUCT* ps, HDC wdc)
 {
 	int textwidth, textheight;
 	glyphcache_getfontsize(&textwidth, &textheight);
 
+	if (!shadow)
+		return;
+
+	HDC dc = CreateCompatibleDC(wdc);
+	SelectObject(dc, shadow);
+
 	int x1 = ps->rcPaint.left / textwidth;
 	x1 -= 1; /* because of overlapping characters */
 	if (x1 < 0)
@@ -247,7 +267,6 @@ static void paint_cb(HWND window, PAINTSTRUCT* ps, HDC dc)
 		y2 = screenheight-1;
 	}
 
-	int state = SaveDC(dc);
 
 	for (int y = y1; y <= y2; y++)
 	{
@@ -271,7 +290,7 @@ static void paint_cb(HWND window, PAINTSTRUCT* ps, HDC dc)
 			{
 				BitBlt(dc, sx+glyph->xoffset, sy+glyph->yoffset,
 					glyph->realwidth, glyph->realheight,
-					glyph->dc, 0, 0, SRCPAINT);
+					glyph->dc, 0, 0, SRCCOPY);
 
 				if (id & DPY_UNDERLINE)
 				{
@@ -312,6 +331,7 @@ static void paint_cb(HWND window, PAINTSTRUCT* ps, HDC dc)
 
 	/* Draw the cursor caret. */
 
+	if (cursorshown)
 	{
 		int x = cursorx*textwidth;
 		int y = cursory*textheight;
@@ -330,7 +350,16 @@ static void paint_cb(HWND window, PAINTSTRUCT* ps, HDC dc)
 	DeleteObject(normalpen);
 	DeleteObject(dimpen);
 	DeleteObject(blackbrush);
-	RestoreDC(dc, state);
+
+	BitBlt(wdc,
+		ps->rcPaint.left, ps->rcPaint.top,
+		ps->rcPaint.right - ps->rcPaint.left,
+		ps->rcPaint.bottom - ps->rcPaint.top,
+		dc,
+		ps->rcPaint.left, ps->rcPaint.top,
+		SRCCOPY);
+
+	DeleteDC(dc);
 }
 
 static void setfont_cb(void)
@@ -469,6 +498,7 @@ static LRESULT CALLBACK window_cb(HWND window, UINT message,
 				create_cb();
 				window_created = true;
 			}
+			resize_shadow(lparam & 0xffff, lparam >> 16);
 			resize_buffer(false);
 			break;
 
@@ -572,6 +602,22 @@ void dpy_init(const char* argv[])
 	read_window_geometry();
 }
 
+static void resize_shadow(int width, int height)
+{
+	if (shadow && (shadowwidth == width) && (shadowheight == height))
+		return;
+	if (shadow)
+		DeleteObject(shadow);
+
+	HDC dc = GetDC(window);
+
+	shadow = CreateCompatibleBitmap(dc, width, height);
+	shadowwidth = width;
+	shadowheight = height;
+
+	ReleaseDC(window, dc);
+}
+
 static void resize_buffer(bool force)
 {
 	RECT rect;
@@ -623,39 +669,6 @@ static void resize_buffer(bool force)
 	InvalidateRect(window, &r, 0);
 }
 
-static void switch_to_full_screen(void)
-{
-	HMONITOR monitor;
-
-	if (window)
-	{
-		monitor = MonitorFromWindow(window, MONITOR_DEFAULTTONEAREST);
-		DestroyWindow(window);
-	}
-	else
-		monitor = MonitorFromWindow(HWND_DESKTOP, MONITOR_DEFAULTTONEAREST);
-
-	MONITORINFO mi;
-	mi.cbSize = sizeof(mi);
-	GetMonitorInfo(monitor, &mi);
-
-	window = CreateWindowExW(
-		WS_EX_TOPMOST,                  /* Extended class style */
-		L"WordGrinder",                 /* Class Name */
-		L"WordGrinder",                 /* Title */
-		WS_POPUP,                       /* Style */
-		mi.rcMonitor.left,              /* x */
-		mi.rcMonitor.top,               /* y */
-		mi.rcMonitor.right - mi.rcMonitor.left, /* width */
-		mi.rcMonitor.bottom - mi.rcMonitor.top, /* height */
-		HWND_DESKTOP,                   /* Parent */
-		NULL,                           /* No menu */
-		GetModuleHandle(NULL),          /* Instance */
-		0);                             /* No special parameters */
-
-	ShowWindow(window, SW_SHOWDEFAULT);
-}
-
 /* Actually invalidates a 3x3 square around the character, to deal with
  * overdraw. */
 static void invalidate_character_at(int x, int y)
@@ -684,7 +697,7 @@ static void insert_item_on_menu(const char* msg, int id, HMENU menu)
 	InsertMenuItem(menu, count+1, TRUE, &mii);
 }
 
-static void switch_to_windowed(void)
+static void create_window(void)
 {
 	if (window)
 		DestroyWindow(window);
@@ -745,6 +758,36 @@ static void switch_to_windowed(void)
 	ShowWindow(window, SW_SHOWDEFAULT);
 }
 
+static void switch_to_full_screen(void)
+{
+	HMONITOR monitor = MonitorFromWindow(window, MONITOR_DEFAULTTONEAREST);
+
+	MONITORINFO mi;
+	mi.cbSize = sizeof(mi);
+	GetMonitorInfo(monitor, &mi);
+
+	GetWindowPlacement(window, &nonFullScreenSize);
+
+	uint32_t style = GetWindowLong(window, GWL_STYLE);
+	SetWindowLong(window, GWL_STYLE, style & ~WS_OVERLAPPEDWINDOW);
+	SetWindowPos(window, HWND_TOP,
+		mi.rcMonitor.left,              /* x */
+		mi.rcMonitor.top,               /* y */
+		mi.rcMonitor.right - mi.rcMonitor.left, /* width */
+		mi.rcMonitor.bottom - mi.rcMonitor.top, /* height */
+		SWP_NOOWNERZORDER | SWP_FRAMECHANGED);
+}
+
+static void switch_to_windowed(void)
+{
+	uint32_t style = GetWindowLong(window, GWL_STYLE);
+	SetWindowLong(window, GWL_STYLE, style | WS_OVERLAPPEDWINDOW);
+	SetWindowPlacement(window, &nonFullScreenSize);
+	SetWindowPos(window, NULL, 0, 0, 0, 0,
+                 SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER |
+                 SWP_NOOWNERZORDER | SWP_FRAMECHANGED);
+}
+
 void dpy_start(void)
 {
 	/* Register our window class. */
@@ -769,7 +812,7 @@ void dpy_start(void)
 		}
 	}
 
-	switch_to_windowed();
+	create_window();
 }
 
 void dpy_shutdown(void)
@@ -813,13 +856,14 @@ void dpy_sync(void)
 	UpdateWindow(window);
 }
 
-void dpy_setcursor(int x, int y)
+void dpy_setcursor(int x, int y, bool shown)
 {
 	invalidate_character_at(cursorx, cursory);
 	invalidate_character_at(x, y);
 
 	cursorx = x;
 	cursory = y;
+	cursorshown = shown;
 
 	UpdateWindow(window);
 }
