@@ -8,6 +8,8 @@
 #include <locale.h>
 #include "globals.h"
 
+extern int luaopen_lpeg (lua_State *L);
+
 static void findlocale(void)
 {
 	static const char* locales[] =
@@ -35,7 +37,14 @@ int main(int argc, const char* argv[])
 	screen_init(argv);
 	word_init();
 	utils_init();
+	filesystem_init();
 	zip_init();
+	#if (LUA_VERSION_NUM < 502)
+		luaopen_lpeg(L);
+	#else
+		luaL_requiref(L, "lpeg", luaopen_lpeg, 1);
+		lua_pop(L, 1);
+	#endif
 
 	script_load_from_table(script_table);
 	script_run(argv);
