@@ -1,7 +1,11 @@
--- © 2008 David Given.
+-- © 2020 David Given.
 -- WordGrinder is licensed under the MIT open source license. See the COPYING
 -- file in this distribution for the full text.
 
+-- Lua 5.3 moved unpack into table.unpack.
+
+unpack = unpack or table.unpack
+
 -- Urrgh, luajit's path defaults to all the wrong places. This is painfully
 -- evil but does at least work.
 
@@ -12,12 +16,6 @@ if jit then
 		";/usr/lib/x86_64-linux-gnu/lua/5.1/?.so;/usr/lib/lua/5.1/?.so;/usr/local/lib/lua/5.1/loadall.so"
 end
 
--- Load the LFS module if needed (Windows has it built in).
-
-if not lfs then
-	lfs = require "lfs"
-end
-
 -- Bit library fallbacks. (LuaJIT vs Lua 5.2 incompatibilities.)
 
 if not bit32 then
@@ -62,3 +60,9 @@ end
 -- Global definitions that the various source files need.
 
 Cmd = {}
+lunamark = {}
+
+BLINK_ON_TIME = 0.8
+BLINK_OFF_TIME = 0.53
+IDLE_TIME = (BLINK_ON_TIME + BLINK_OFF_TIME) * 5
+
