@@ -89,6 +89,7 @@ DocumentSetClass =
 
 		self:touch()
 		RebuildDocumentsMenu(self.documents)
+		return document
 	end,
 
 	moveDocumentIndexTo = function(self, name, targetIndex)
@@ -146,6 +147,7 @@ DocumentSetClass =
 		end
 
 		self.current = Document
+		Document:renumber()
 		ResizeScreen()
 	end,
 
@@ -271,6 +273,25 @@ DocumentClass =
 	touch = function(self)
 		FireEvent(Event.DocumentModified, self)
 	end,
+
+	renumber = function(self)
+		local wc = 0
+		local pn = 1
+
+		for _, p in ipairs(self) do
+			wc = wc + #p
+
+			local style = DocumentStyles[p.style]
+			if style.numbered then
+				p.number = pn
+				pn = pn + 1
+			elseif not style.list then
+				pn = 1
+			end
+		end
+
+		self.wordcount = wc
+	end
 }
 
 ParagraphClass =
@@ -288,15 +309,35 @@ ParagraphClass =
 		self.lines = nil
 		self.wrapwidth = nil
 		self.xs = nil
+		self.sentences = nil
 	end,
 
 	wrap = function(self, width)
+		local sentences = self.sentences
+		if (sentences == nil) then
+			local issentence = true
+			sentences = {}
+			for wn, word in ipairs(self) do
+				if issentence then
+					sentences[wn] = true
+					issentence = false
+				end
+
+				if word:find("%.$") then
+					issentence = true
+				end
+			end
+			sentences[#self] = true
+			self.sentences = sentences
+		end
+
 		width = width or Document.wrapwidth
 		if (self.wrapwidth ~= width) then
 			local lines = {}
 			local line = {wn = 1}
 			local w = 0
 			local xs = {}
+			local fullstopspaces = WantFullStopSpaces()
 			self.xs = xs
 
 			width = width - self:getIndentOfLine(1)
@@ -305,6 +346,11 @@ ParagraphClass =
 				-- get width of word (including space)
 				local ww = GetStringWidth(word) + 1
 
+				-- add an extra space if the user asked for it
+				if fullstopspaces and word:find("%.$") then
+					ww = ww + 1
+				end
+
 				xs[wn] = w
 				w = w + ww
 				if (w >= width) then
@@ -340,7 +386,8 @@ ParagraphClass =
 			local payload = {
 				word = w,
 				ostyle = ostyle,
-				cstyle = cstyle
+				cstyle = cstyle,
+				firstword = self.sentences[wn]
 			}
 			FireEvent(Event.DrawWord, payload)
 
@@ -398,7 +445,8 @@ ParagraphClass =
 			local payload = {
 				word = self[w],
 				ostyle = ostyle,
-				cstyle = cstyle
+				cstyle = cstyle,
+				firstword = self.sentences[wn]
 			}
 			FireEvent(Event.DrawWord, payload)
 
@@ -526,10 +574,21 @@ function GetWordSimpleText(s)
 	s = s:gsub('[~#&^$"<>]+', "")
 	s = s:gsub("^[.'([{]+", "")
 	s = s:gsub("[',.!?:;)%]}]+$", "")
-	s = s:lower()
 	return s
 end
 
+function OnlyFirstCharIsUppercase(s)
+    -- Return true if only first character is uppercase
+    local first_char = s:sub(0, 1)
+    if first_char:upper() == first_char then
+        local remaining_chars = s:sub(2, s:len())
+        if remaining_chars:lower() == remaining_chars then
+            return true
+        end
+    end
+    return false
+end
+
 function UpdateDocumentStyles()
 	local plaintext =
 	{
@@ -588,6 +647,16 @@ function UpdateDocumentStyles()
 			below = 1,
 			indent = 4,
 			bullet = "-",
+			list = true,
+		},
+		{
+			desc = "List item with number",
+			name = "LN",
+			above = 1,
+			below = 1,
+			indent = 4,
+			numbered = true,
+			list = true,
 		},
 		{
 			desc = "List item without bullet",
@@ -595,6 +664,7 @@ function UpdateDocumentStyles()
 			above = 1,
 			below = 1,
 			indent = 4,
+			list = true,
 		},
 		{
 			desc = "Indented text, run together",
@@ -632,9 +702,7 @@ function CreateDocumentSet()
 	{
 		fileformat = FILEFORMAT,
 		statusbar = true,
-		idletime = 3,
 		documents = {},
-		styles = DocumentStyles,
 		addons = {},
 	}
 
