@@ -471,7 +471,7 @@ end
 
 function loadfromstreamt(fp)
 	local data = CreateDocumentSet()
-	data.menu = CreateMenu()
+	data.menu = CreateMenuBindings()
 	data.documents = {}
 
 	while true do
@@ -641,7 +641,7 @@ function Cmd.LoadDocumentSet(filename)
 		FireEvent(Event.DocumentUpgrade, fileformat, FILEFORMAT)
 
 		DocumentSet.fileformat = FILEFORMAT
-		DocumentSet.menu = CreateMenu()
+		DocumentSet.menu = CreateMenuBindings()
 	end
 	FireEvent(Event.RegisterAddons)
 	DocumentSet:touch()
@@ -661,6 +661,11 @@ function Cmd.LoadDocumentSet(filename)
 			"Also, all keybindings defined in this file will get reset "..
 			"to their default values.")
 	end
+
+	-- The document is NOT dirty immediately after a load.
+	
+	DocumentSet.changed = false
+
 	return true
 end
 
@@ -726,4 +731,17 @@ function UpgradeDocument(oldversion)
         end
 		DocumentSet.styles = nil
 	end
+
+	-- Upgrade version 7 to 8.
+	
+	if (oldversion < 8) then
+		-- This version added the LN paragraph style type; documents are forwards
+		-- compatible but not backward compatible.
+		
+		-- A bug on 0.7.2 meant that the styles were still exported in
+		-- WordGrinder files, even though they were never used.
+
+		DocumentSet.styles = nil
+		DocumentSet.idletime = nil
+	end
 end
