@@ -3,6 +3,7 @@
 -- file in this distribution for the full text.
 
 local int = math.floor
+local min = math.min
 local Write = wg.write
 local GotoXY = wg.gotoxy
 local ClearArea = wg.cleararea
@@ -13,6 +14,11 @@ local SetUnderline = wg.setunderline
 local SetReverse = wg.setreverse
 local SetDim = wg.setdim
 local GetStringWidth = wg.getstringwidth
+local ShowCursor = wg.showcursor
+local HideCursor = wg.hidecursor
+local Sync = wg.sync
+
+local BLINK_TIME = 0.8
 
 local messages = {}
 local leftpadding = 0
@@ -46,12 +52,23 @@ local function drawmargin(y, pn, p)
 		end
 	end
 
-	local bullet = DocumentStyles[p.style].bullet
-	if bullet then
-		local w = GetStringWidth(bullet) + 1
-		local i = DocumentStyles[p.style].indent
+	local style = DocumentStyles[p.style]
+	local function drawbullet(n)
+		local w = GetStringWidth(n) + 1
+		local i = style.indent
 		if (i >= w) then
-			Write(leftpadding + Document.margin + i - w, y, bullet)
+			Write(leftpadding + Document.margin + i - w, y, n)
+		end
+	end
+
+	local bullet = style.bullet
+	if bullet then
+		drawbullet(bullet)
+	else
+		local numbered = style.numbered
+		if numbered then
+			local n = tostring(p.number or 0).."."
+			drawbullet(n)
 		end
 	end
 end
@@ -285,18 +302,38 @@ function RedrawScreen()
 	FireEvent(Event.Redraw)
 end
 
+function GetCharWithBlinkingCursor(timeout)
+	ShowCursor()
+
+	timeout = timeout or 1E10
+	local shown = true
+	while timeout > 0 do
+		local t = shown and BLINK_ON_TIME or BLINK_OFF_TIME
+		t = min(t, timeout)
+		local c = wg.getchar(t)
+		if (c ~= "KEY_TIMEOUT") then
+			ShowCursor();
+			return c
+		end
+
+		shown = not shown
+		local cb = shown and ShowCursor or HideCursor
+		cb()
+
+		timeout = timeout - t
+	end
+	
+	return "KEY_TIMEOUT"
+end
+
 -----------------------------------------------------------------------------
--- Maintains the word count field in the current document.
+-- Does assorted fast updates in the current document on changes:
+--   - word count
+--   - numbered paragraph styles
 
 do
 	local function cb(event, token)
-		local wc = 0
-
-		for _, p in ipairs(Document) do
-			wc = wc + #p
-		end
-
-		Document.wordcount = wc
+		Document:renumber()
 	end
 
 	AddEventListener(Event.Changed, cb)
