@@ -10,7 +10,7 @@ Cmd.InsertStringIntoParagraph("bold")
 Cmd.SetStyle("b")
 Cmd.SetMark()
 Cmd.InsertStringIntoParagraph("italic")
-Cmd.SetStyle("b")
+Cmd.SetStyle("i")
 Cmd.SetMark()
 Cmd.InsertStringIntoParagraph("underline")
 Cmd.SetStyle("u")
@@ -25,5 +25,5 @@ four bolditalicunderline
 heading
 ]]
 
-local output = Cmd.ExportToTextString(Document)
+local output = Cmd.ExportToTextString()
 AssertEquals(expected, output)
