@@ -42,7 +42,7 @@ AssertTableEquals({"\016bar"}, Document[2])
 Cmd.InsertStringIntoParagraph("o bar")
 Cmd.GotoEndOfWord()
 Cmd.DeleteWord()
-AssertEquals(2, Document.cw)
-AssertEquals(1, Document.co)
-AssertTableEquals({"o", ""}, Document[2])
+AssertEquals(1, Document.cw)
+AssertEquals(2, Document.co)
+AssertTableEquals({"o"}, Document[2])
 
