@@ -32,11 +32,23 @@ ExportToString(Document, callback)
 
 AssertTableAndPropertiesEquals(
 	{
-		prologue = {{}},
-		paragraph_start = {{"P"}, {"P"}, {"P"}},
-		text = {{"one"}, {""}, {"two"}},
-		paragraph_end = {{"P"}, {"P"}, {"P"}},
-		epilogue = {{}}
-	},
+		["paragraph_start"] = {
+			{{"one", ["style"]="P"}},
+			{{"\16", ["style"]="P"}},
+			{{"two", ["style"]="P"}}
+		},
+		["paragraph_end"] = {
+			{{"one", ["style"]="P"}},
+			{{"\16", ["style"]="P"}},
+			{{"two", ["style"]="P"}}
+		},
+		["prologue"] = {{}},
+		["text"] = {
+			{"one"},
+			{""},
+			{"two"}
+		},
+		["epilogue"] = {{}}
+	}, 
 	result)
 
